@@ -23,8 +23,6 @@
  *
  * SPDX-License-Identifier: MIT
  */
-#include "ares_setup.h"
-#include "ares.h"
 #include "ares_private.h"
 #include "ares_event.h"
 
@@ -128,7 +126,7 @@ ares_status_t ares_event_update(ares_event_t **event, ares_event_thread_t *e,
     }
 
     if (ares__llist_insert_last(e->ev_updates, ev) == NULL) {
-      ares_free(ev); /* LCOV_EXCL_LINE: OutOfMemory */
+      ares_free(ev);      /* LCOV_EXCL_LINE: OutOfMemory */
       return ARES_ENOMEM; /* LCOV_EXCL_LINE: OutOfMemory */
     }
   }
@@ -264,6 +262,35 @@ static void ares_event_process_updates(ares_event_thread_t *e)
   }
 }
 
+static void ares_event_thread_cleanup(ares_event_thread_t *e)
+{
+  /* Manually free any updates that weren't processed */
+  if (e->ev_updates != NULL) {
+    ares__llist_node_t *node;
+
+    while ((node = ares__llist_node_first(e->ev_updates)) != NULL) {
+      ares_event_destroy_cb(ares__llist_node_claim(node));
+    }
+    ares__llist_destroy(e->ev_updates);
+    e->ev_updates = NULL;
+  }
+
+  if (e->ev_sock_handles != NULL) {
+    ares__htable_asvp_destroy(e->ev_sock_handles);
+    e->ev_sock_handles = NULL;
+  }
+
+  if (e->ev_cust_handles != NULL) {
+    ares__htable_vpvp_destroy(e->ev_cust_handles);
+    e->ev_cust_handles = NULL;
+  }
+
+  if (e->ev_sys != NULL && e->ev_sys->destroy != NULL) {
+    e->ev_sys->destroy(e);
+    e->ev_sys = NULL;
+  }
+}
+
 static void *ares_event_thread(void *arg)
 {
   ares_event_thread_t *e = arg;
@@ -274,16 +301,19 @@ static void *ares_event_thread(void *arg)
     const struct timeval *tvout;
     unsigned long         timeout_ms = 0; /* 0 = unlimited */
 
+    ares_event_process_updates(e);
+
+    /* Don't hold a mutex while waiting on events or calling into anything
+     * that might require a c-ares channel lock since a callback could be
+     * triggered cross-thread */
+    ares__thread_mutex_unlock(e->mutex);
+
     tvout = ares_timeout(e->channel, NULL, &tv);
     if (tvout != NULL) {
       timeout_ms =
         (unsigned long)((tvout->tv_sec * 1000) + (tvout->tv_usec / 1000) + 1);
     }
 
-    ares_event_process_updates(e);
-
-    /* Don't hold a mutex while waiting on events */
-    ares__thread_mutex_unlock(e->mutex);
     e->ev_sys->wait(e, timeout_ms);
 
     /* Each iteration should do timeout processing */
@@ -295,14 +325,16 @@ static void *ares_event_thread(void *arg)
     ares__thread_mutex_lock(e->mutex);
   }
 
+  /* Lets cleanup while we're in the thread itself */
+  ares_event_thread_cleanup(e);
+
   ares__thread_mutex_unlock(e->mutex);
+
   return NULL;
 }
 
 static void ares_event_thread_destroy_int(ares_event_thread_t *e)
 {
-  ares__llist_node_t *node;
-
   /* Wake thread and tell it to shutdown if it exists */
   ares__thread_mutex_lock(e->mutex);
   if (e->isup) {
@@ -313,26 +345,14 @@ static void ares_event_thread_destroy_int(ares_event_thread_t *e)
 
   /* Wait for thread to shutdown */
   if (e->thread) {
-    ares__thread_join(e->thread, NULL);
+    void *rv = NULL;
+    ares__thread_join(e->thread, &rv);
     e->thread = NULL;
   }
 
-  /* Manually free any updates that weren't processed */
-  while ((node = ares__llist_node_first(e->ev_updates)) != NULL) {
-    ares_event_destroy_cb(ares__llist_node_claim(node));
-  }
-  ares__llist_destroy(e->ev_updates);
-  e->ev_updates = NULL;
-
-  ares__htable_asvp_destroy(e->ev_sock_handles);
-  e->ev_sock_handles = NULL;
-
-  ares__htable_vpvp_destroy(e->ev_cust_handles);
-  e->ev_cust_handles = NULL;
-
-  if (e->ev_sys && e->ev_sys->destroy) {
-    e->ev_sys->destroy(e);
-  }
+  /* If the event thread ever got to the point of starting, this is a no-op
+   * as it runs this same cleanup when it shuts down */
+  ares_event_thread_cleanup(e);
 
   ares__thread_mutex_destroy(e->mutex);
   e->mutex = NULL;
@@ -349,6 +369,8 @@ void ares_event_thread_destroy(ares_channel_t *channel)
   }
 
   ares_event_thread_destroy_int(e);
+  channel->sock_state_cb_data = NULL;
+  channel->sock_state_cb = NULL;
 }
 
 static const ares_event_sys_t *ares_event_fetch_sys(ares_evsys_t evsys)
@@ -391,22 +413,23 @@ static const ares_event_sys_t *ares_event_fetch_sys(ares_evsys_t evsys)
 
     /* case ARES_EVSYS_DEFAULT: */
     default:
-#if defined(USE_WINSOCK)
-      return &ares_evsys_win32;
-#elif defined(HAVE_KQUEUE)
-      return &ares_evsys_kqueue;
-#elif defined(HAVE_EPOLL)
-      return &ares_evsys_epoll;
-#elif defined(HAVE_POLL)
-      return &ares_evsys_poll;
-#elif defined(HAVE_PIPE)
-      return &ares_evsys_select;
-#else
       break;
-#endif
   }
 
+    /* default */
+#if defined(USE_WINSOCK)
+  return &ares_evsys_win32;
+#elif defined(HAVE_KQUEUE)
+  return &ares_evsys_kqueue;
+#elif defined(HAVE_EPOLL)
+  return &ares_evsys_epoll;
+#elif defined(HAVE_POLL)
+  return &ares_evsys_poll;
+#elif defined(HAVE_PIPE)
+  return &ares_evsys_select;
+#else
   return NULL;
+#endif
 }
 
 ares_status_t ares_event_thread_init(ares_channel_t *channel)
@@ -421,25 +444,25 @@ ares_status_t ares_event_thread_init(ares_channel_t *channel)
   e->mutex = ares__thread_mutex_create();
   if (e->mutex == NULL) {
     ares_event_thread_destroy_int(e); /* LCOV_EXCL_LINE: OutOfMemory */
-    return ARES_ENOMEM; /* LCOV_EXCL_LINE: OutOfMemory */
+    return ARES_ENOMEM;               /* LCOV_EXCL_LINE: OutOfMemory */
   }
 
   e->ev_updates = ares__llist_create(NULL);
   if (e->ev_updates == NULL) {
     ares_event_thread_destroy_int(e); /* LCOV_EXCL_LINE: OutOfMemory */
-    return ARES_ENOMEM; /* LCOV_EXCL_LINE: OutOfMemory */
+    return ARES_ENOMEM;               /* LCOV_EXCL_LINE: OutOfMemory */
   }
 
   e->ev_sock_handles = ares__htable_asvp_create(ares_event_destroy_cb);
   if (e->ev_sock_handles == NULL) {
     ares_event_thread_destroy_int(e); /* LCOV_EXCL_LINE: OutOfMemory */
-    return ARES_ENOMEM; /* LCOV_EXCL_LINE: OutOfMemory */
+    return ARES_ENOMEM;               /* LCOV_EXCL_LINE: OutOfMemory */
   }
 
   e->ev_cust_handles = ares__htable_vpvp_create(NULL, ares_event_destroy_cb);
   if (e->ev_cust_handles == NULL) {
     ares_event_thread_destroy_int(e); /* LCOV_EXCL_LINE: OutOfMemory */
-    return ARES_ENOMEM; /* LCOV_EXCL_LINE: OutOfMemory */
+    return ARES_ENOMEM;               /* LCOV_EXCL_LINE: OutOfMemory */
   }
 
   e->channel = channel;
@@ -447,7 +470,7 @@ ares_status_t ares_event_thread_init(ares_channel_t *channel)
   e->ev_sys  = ares_event_fetch_sys(channel->evsys);
   if (e->ev_sys == NULL) {
     ares_event_thread_destroy_int(e); /* LCOV_EXCL_LINE: UntestablePath */
-    return ARES_ENOTIMP; /* LCOV_EXCL_LINE: UntestablePath */
+    return ARES_ENOTIMP;              /* LCOV_EXCL_LINE: UntestablePath */
   }
 
   channel->sock_state_cb      = ares_event_thread_sockstate_cb;
